// vim: set fdm=marker ts=4 sw=4:

#pragma warning(disable: 4819)

#include <MyGDB.h>

Config::Config() 
{ 
	sXml = wxT("");

	fileName = exePath + "/mygdb.cfg";

	if ( wxFileName::FileExists(fileName) )
	{
		Open();
	}
	else
	{
		xml = new wxXmlDocument();
		xml->SetEncoding(_T("UTF-8"));
		xml->SetVersion(_T("1.0"));

		root = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("MYGDB"));
		xml->SetRoot(root);

		// version
		SetProperty("VERSION", "value", "1.0");

		// creation
		wxDateTime time = wxDateTime::Now();
		SetProperty("CREATION", "value", 
				time.Format("%c", wxDateTime::CET).c_str());
		SetProperty("NAME", "value", "MyGDB Configuration");
		
		// invoke application
		SetProperty("INVOKE/EXECUTABLELOCATION", "value", "");
		SetProperty("INVOKE/SOURCELOCATION", "value", "");
		SetProperty("INVOKE/GDBLOCATION", "value", "");
		SetProperty("INVOKE/ARGS", "value", "");
		
		SetProperty("DEBUG/BREAKPOINT", "value", "");

		// editor general
		SetProperty("EDITOR/GENERAL/USETABCHAR", "value", "true");
		SetProperty("EDITOR/GENERAL/TABINDENTS", "value", "false");
		SetProperty("EDITOR/GENERAL/TABSIZE", "value", "4");
		SetProperty("EDITOR/GENERAL/AUTOINDENT", "value", "true");
		SetProperty("EDITOR/GENERAL/SMARTINDENT", "value", "true");
		SetProperty("EDITOR/GENERAL/INDENTGUIDES", "value", "true");
		// 0: file name only, 1: relative file name, 2: full path
		SetProperty("EDITOR/GENERAL/TITLE", "value", "0");
		SetProperty("EDITOR/GENERAL/WORDWRAP", "value", "true");
		SetProperty("EDITOR/GENERAL/LINENUMBER", "value", "true");
		SetProperty("EDITOR/GENERAL/LINECARET", "value", "true");
		//SetProperty("EDITOR/GENERAL/SHOWHELP", "value", "false");
		SetProperty("EDITOR/GENERAL/SHOWVALUE", "value", "true");
		SetProperty("EDITOR/GENERAL/SHOWEDGE", "value", "false");
		SetProperty("EDITOR/GENERAL/EDGECOLUMN", "value", "80");
		SetProperty("EDITOR/GENERAL/RULER", "value", "false");
		SetProperty("EDITOR/GENERAL/CODEEXPLORERUPDATETIME", "value", "2000");

		Save();
	}
}

Config::~Config() { }

void Config::Save (void)
{
	xml->Save(fileName);
	sXml = Read(fileName);
}

bool Config::Open(void)
{
	xml = new wxXmlDocument();
	if ( !xml->Load(fileName) ) 
	{
		return false;
	}
	
	if (xml->GetRoot()->GetName() != _T("MYGDB")) 
	{
		wxMessageDialog(NULL, 
			_T("Unknown MyGDB configuration file"),
			"MyGDB", wxOK|wxICON_ERROR).ShowModal();
		return false;
	}

	root = xml->GetRoot();
	
	// -------------
	sXml = Read(fileName);

	return true;
}

wxString Config::Read(wxString fileName)
{
	wxString buffer = wxT("");

	wxFile* file = new wxFile();
	if ( file->Open(fileName, wxFile::read) ) {
		unsigned long fileSize = file->Length();

		char *pBuf = (char *)malloc(fileSize+1);
		size_t nResult = file->Read( pBuf, fileSize );

		if ( wxInvalidOffset == (int)nResult ) {
			wxMessageBox(wxString::Format("Error reading %s file.", 
						fileName.c_str()), wxT("MyGDB"), wxICON_ERROR);
			return buffer;
		}

		pBuf[fileSize] = 0;
		buffer = pBuf;
		file->Close();
		free(pBuf);
	}
	delete file;

	return buffer;
}

bool Config::Changed(void)
{
	wxString buffer;

	wxStringOutputStream stream;
	xml->Save(stream);
	buffer = stream.GetString();
	stream.Close();

	if ( sXml == buffer )
	{
		return false;
	}
	else
	{
		return true;
	}
}

void Config::Backup(void)
{
	wxStringOutputStream stream;
	xml->Save(stream);
	backup = stream.GetString();
	stream.Close();
}

void Config::Restore(void)
{
	wxStringInputStream stream(backup);
	delete xml;
	xml = new wxXmlDocument();
	xml->Load(stream);
	root = xml->GetRoot();
}

wxXmlNode* Config::SetNode(wxString nodePath)
{
	wxXmlNode* pNode = root;	// parent node

	wxStringTokenizer tkz(nodePath, wxT("/"));
	while ( tkz.HasMoreTokens() ) 
	{
		wxString token = tkz.GetNextToken();
		
		wxXmlNode* node = pNode->GetChildren();
	
		int exist = false;
		while (node) 
		{
			if ( node->GetName() == token ) 
			{
				pNode = node;
				exist = true;
				break;
			}
			node = node->GetNext();
		}

		if ( exist == false ) 
		{
			wxXmlNode *childNode = new wxXmlNode (wxXML_ELEMENT_NODE, token);
			pNode->AddChild(childNode);
			pNode = childNode;
		}
	}

	return pNode;
}

bool Config::SetProperty(wxString nodePath, wxString property, wxString value)
{
	wxXmlNode *node = SetNode(nodePath);
	wxXmlProperty* prop = node->GetProperties();
	
	bool exist=false;	
	while (prop) 
	{
		if ( prop->GetName() == property ) 
		{
			prop->SetValue(value);
			exist=true;
		}
		prop = prop->GetNext();
	}

	if (exist==false)
	{
		node->AddProperty(property, value);
	}

	return true;
}

bool Config::GetPropertyBool(wxString nodePath, wxString property)
{
	wxString value = GetProperty(nodePath, property);
	if ( value == wxT("true") ) 
		return true;
	else 
		return false;
}

long Config::GetPropertyInt(wxString nodePath, wxString property)
{
	long intValue; 
	wxString value = GetProperty(nodePath, property);
	value.ToLong(&intValue, 10);
	return intValue;
}

wxString Config::GetProperty(wxString nodePath, wxString property)
{
	wxXmlNode *node = SetNode(nodePath);
	wxXmlProperty* prop = node->GetProperties();
		
	while (prop) 
	{
		if ( prop->GetName() == property ) 
		{
			return prop->GetValue();
		}
		prop = prop->GetNext();
	}
		
	if ( nodePath == "EDITOR/GENERAL/USETABCHAR" ) return wxT("true");
	if ( nodePath == "EDITOR/GENERAL/TABINDENTS") return wxT("false");
	if ( nodePath == "EDITOR/GENERAL/TABSIZE" ) return wxT("4");
	if ( nodePath == "EDITOR/GENERAL/AUTOINDENT" ) return wxT("true");
	if ( nodePath == "EDITOR/GENERAL/SMARTINDENT" ) return wxT("true");
	if ( nodePath == "EDITOR/GENERAL/INDENTGUIDES" ) return wxT("true");
	// 0: file name only, 1: relative file name, 2: full path
	if ( nodePath == "EDITOR/GENERAL/TITLE" ) return wxT("0");
	if ( nodePath == "EDITOR/GENERAL/WORDWRAP" ) return wxT("true");
	if ( nodePath == "EDITOR/GENERAL/LINENUMBER" ) return wxT("true");
	if ( nodePath == "EDITOR/GENERAL/LINECARET" ) return wxT("true");
	//if ( nodePath == "EDITOR/GENERAL/SHOWHELP" ) return wxT("false");
	if ( nodePath == "EDITOR/GENERAL/SHOWVALUE" ) return wxT("true");
	if ( nodePath == "EDITOR/GENERAL/SHOWEDGE" ) return wxT("true");
	if ( nodePath == "EDITOR/GENERAL/EDGECOLUMN" ) return wxT("80");
	if ( nodePath == "EDITOR/GENERAL/RULER" ) return wxT("false");
	if ( nodePath == "EDITOR/GENERAL/CODEEXPLORERUPDATETIME" ) return wxT("2000");
	if ( nodePath == "EDITOR/STYLE" ) return wxT("Default");

	return wxT("");
}
