// vim: set ts=4 sw=4:

#include <MyGDB.h>
#include "xpm/mygdb.xpm"
#include "about_bitmap.h"

wxString exePath; 	// MyGDB program path

bool m_started = false;

class MyGdbApp : public wxApp
{
public:
    bool OnInit();
};

DECLARE_APP(MyGdbApp)
IMPLEMENT_APP(MyGdbApp)

bool MyGdbApp::OnInit()
{
	MyGdbFrame *mainFrame = new MyGdbFrame(NULL,
		 wxID_ANY, wxT("MyGDB"), 
		 wxDefaultPosition, wxSize(1000, 700),
		 wxDEFAULT_FRAME_STYLE );
    SetTopWindow(mainFrame);
    mainFrame->Show();

    return true;
}

BEGIN_EVENT_TABLE(MyGdbFrame, wxAuiMDIParentFrame)
	EVT_UPDATE_UI(wxID_ANY, MyGdbFrame::OnUpdateUI)
	
    EVT_CLOSE(MyGdbFrame::OnCloseApp)
    EVT_MENU(wxID_EXIT, MyGdbFrame::OnExit)
	EVT_SIZE(MyGdbFrame::OnSize)

	// -----------
    EVT_MENU(ID_INVOKE_APP, MyGdbFrame::OnInvokeApp)
	
	// -----------
    EVT_MENU(ID_OPEN_FILE, MyGdbFrame::OnOpenFile)

	// -----------
    EVT_MENU(ID_VIEW_CONSOLE, MyGdbFrame::OnViewConsole)
    EVT_MENU(ID_VIEW_WATCH, MyGdbFrame::OnViewWatch)
    EVT_MENU(ID_VIEW_CALLSTACK, MyGdbFrame::OnViewCallStack)
    EVT_MENU(ID_VIEW_BREAKPOINT, MyGdbFrame::OnViewBreakpoint)
    EVT_MENU(ID_VIEW_REGISTER, MyGdbFrame::OnViewRegister)
    EVT_MENU(ID_VIEW_MEMORY, MyGdbFrame::OnViewMemory)

	// -----------
    EVT_MENU(ID_START, MyGdbFrame::OnStart)
    EVT_MENU(ID_ABORT, MyGdbFrame::OnAbort)
    EVT_MENU(ID_STOP, MyGdbFrame::OnStop)
    
	// next
	EVT_MENU(ID_STEP_OVER, MyGdbFrame::OnStepOver)
	// si
	EVT_MENU(ID_STEP_INTO, MyGdbFrame::OnStepInto)
	// finish
	EVT_MENU(ID_STEP_OUT, MyGdbFrame::OnStepOut)
	// continue
	EVT_MENU(ID_GO, MyGdbFrame::OnGo)
	// continue
	EVT_MENU(ID_RUN_TO_CURSOR, MyGdbFrame::OnRunToCursor)
    
	// -----------
	EVT_MENU(wxID_ABOUT, MyGdbFrame::OnAbout)
END_EVENT_TABLE()

MyGdbFrame::MyGdbFrame(wxWindow* parent,
	 wxWindowID id,
	 const wxString& title,
	 const wxPoint& pos,
	 const wxSize& size,
	 long style)
	: wxAuiMDIParentFrame(parent, id, title, pos, size, style)
{
    m_mgr.SetManagedWindow(this);

	wxInitAllImageHandlers();

#ifdef __MINGW32__
	wxChar _drv[MAX_PATH], _path[MAX_PATH];
	wxChar fname[MAX_PATH], ext[MAX_PATH], temp[MAX_PATH];

	GetModuleFileName(NULL, temp, 1024);
	_splitpath(temp, _drv, _path, fname, ext);

	wxFileName path(wxString::Format("%s%s", _drv, _path));
	exePath = path.GetPath();
#else
	char procname[FILENAME_MAX];
	int len = readlink("/proc/self/exe", procname, FILENAME_MAX - 1);
	procname[len] = '\0';
	wxFileName exe(procname);
	exePath = exe.GetPath();
#endif

	exePath.Replace(wxT("\\"), wxT("/"), true);
    
	// ----------------------
	m_menuBar = new wxMenuBar();
    SetMenuBar(m_menuBar);
	
	wxMenu* menu = new wxMenu;
	wxMenuItem *item;
	
	item = new wxMenuItem(menu, ID_INVOKE_APP, wxT("&Invoke Application\tCtrl+I"));
	item->SetBitmap(GET_BITMAP("application"));
	menu->Append(item);
	menu->AppendSeparator();
	item = new wxMenuItem(menu, wxID_EXIT, wxT("E&xit\tCtrl+X"));
	menu->Append(item);
	m_menuBar->Append(menu, _("&Invoke"));
	
	// ------------------------
	menu = new wxMenu;
	item = new wxMenuItem(menu, ID_OPEN_FILE, wxT("&Open Source\tCtrl+O"));
	item->SetBitmap(GET_BITMAP("file_open"));
	menu->Append(item);
	m_menuBar->Append(menu, _("&File"));
	
	// ----------------------
	menu = new wxMenu;
	
	item = new wxMenuItem(menu, ID_START, wxT("Start\tCtrl+F5"));
	item->SetBitmap(GET_BITMAP("start"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_ABORT, wxT("Abort\tCtrl+F6"));
	item->SetBitmap(GET_BITMAP("abort"));
	menu->Append(item);
	menu->AppendSeparator();
	item = new wxMenuItem(menu, ID_GO, wxT("Continue\tF8"));
	item->SetBitmap(GET_BITMAP("go"));
	item = new wxMenuItem(menu, ID_STEP_INTO, wxT("Step Into\tF11"));
	item->SetBitmap(GET_BITMAP("step_into"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_STEP_OUT, wxT("Step Out\tShift+F11"));
	item->SetBitmap(GET_BITMAP("step_out"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_STEP_OVER, wxT("Step Over\tF10"));
	item->SetBitmap(GET_BITMAP("step_over"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_RUN_TO_CURSOR, wxT("Run To Cursor\tCtrl+F10"));
	item->SetBitmap(GET_BITMAP("run_to_cursor"));
	menu->Append(item);
	menu->AppendSeparator();
	item = new wxMenuItem(menu, ID_STOP, wxT("Stop\tCtrl+F9"));
	item->SetBitmap(GET_BITMAP("stop"));
	menu->Append(item);
	m_menuBar->Append(menu, _("&Debugger"));

	// ------------------------
	menu = new wxMenu;
	SetWindowMenu(menu);
	
	item = new wxMenuItem(menu, ID_VIEW_CONSOLE, wxT("Console Window"));
	menu->Append(item);
	menu->AppendSeparator();
	item = new wxMenuItem(menu, ID_VIEW_WATCH, wxT("Watch Window"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_VIEW_CALLSTACK, wxT("Backtrace Window"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_VIEW_BREAKPOINT, wxT("Breakpoint Window"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_VIEW_REGISTER, wxT("Register Window"));
	menu->Append(item);
	item = new wxMenuItem(menu, ID_VIEW_MEMORY, wxT("Memory Window"));
	menu->Append(item);
	
	// ------------------------
	menu = new wxMenu;
	item = new wxMenuItem(menu, wxID_ABOUT, wxT("About"));
	menu->Append(item);
	m_menuBar->Append(menu, _("&Help"));
	
	// ----------------------	
	m_config = new Config();

#ifdef __MINGW32__
	SetFont(GetDockArt()->GetFont(wxAUI_DOCKART_CAPTION_FONT));
#endif
	
	unsigned int flags = m_mgr.GetFlags();
	flags &= ~wxAUI_MGR_TRANSPARENT_HINT;
	flags &= ~wxAUI_MGR_RECTANGLE_HINT;
	flags &= ~wxAUI_MGR_ALLOW_ACTIVE_PANE;
	m_mgr.SetFlags(flags);
	m_mgr.SetFlags(m_mgr.GetFlags() ^ wxAUI_MGR_RECTANGLE_HINT);
	m_mgr.SetFlags(m_mgr.GetFlags() ^ wxAUI_MGR_ALLOW_ACTIVE_PANE);
	m_mgr.Update();

	// define dock frame style
	m_mgr.SetArtProvider(new MyAuiDockArt());

    //set frame icon
    SetIcon(wxIcon(mygdb_xpm));

	// status bar
	// ---------------------
	m_statusBar = new wxStatusBar(this, -1);
	m_statusBar->SetFieldsCount(3);
	int width[3] = { -1, 150, 150 };
	m_statusBar->SetStatusWidths(3, width);
	
	SetStatusBar(m_statusBar);

	// ----------------------- toolbar
	// -- file
    m_fileToolBar = new wxAuiToolBar(this, 
		wxID_ANY, wxDefaultPosition, wxDefaultSize, 
		wxAUI_TB_DEFAULT_STYLE );
    m_fileToolBar->SetToolBitmapSize(wxSize(16,16));

    m_fileToolBar->AddTool(ID_INVOKE_APP, 
			wxEmptyString, GET_BITMAP("application"), 
			wxT("Invoke Application"));
    m_fileToolBar->AddTool(ID_OPEN_FILE, 
			wxEmptyString, GET_BITMAP("file_open"), 
			wxT("Open Source"));
    
	m_fileToolBar->Realize();

	// -- debug
    m_debugToolBar = new wxAuiToolBar(this, 
		wxID_ANY, wxDefaultPosition, wxDefaultSize, wxAUI_TB_DEFAULT_STYLE );
    m_debugToolBar->SetToolBitmapSize(wxSize(16,16));

    m_debugToolBar->AddTool(ID_START, 
			wxEmptyString, GET_BITMAP("start"), wxT("Debug (Ctrl+F5)"));
    m_debugToolBar->AddSeparator();
    m_debugToolBar->AddTool(ID_ABORT, 
			wxEmptyString, GET_BITMAP("abort"), wxT("Abort (Ctrl+F6)"));

    m_debugToolBar->AddSeparator();

    m_debugToolBar->AddTool(ID_GO, wxEmptyString, 
			GET_BITMAP("go"), wxT("Continue (F8)"));
    m_debugToolBar->AddTool(ID_STEP_INTO, wxEmptyString, 
			GET_BITMAP("step_into"), wxT("Step Into (F11)"));
	m_debugToolBar->AddTool(ID_STEP_OUT, wxEmptyString, 
			GET_BITMAP("step_out"), wxT("Step Out (Shift+F11)"));
	m_debugToolBar->AddTool(ID_STEP_OVER, wxEmptyString, 
			GET_BITMAP("step_over"), wxT("Step Over (F10)"));
	m_debugToolBar->AddTool(ID_RUN_TO_CURSOR, wxEmptyString, 
			GET_BITMAP("run_to_cursor"), wxT("Run To Cursor (Ctrl+F10)"));
    
	m_debugToolBar->AddSeparator();
	
	m_debugToolBar->AddTool(ID_STOP, wxEmptyString, 
			GET_BITMAP("stop"), wxT("Stop (Ctrl+F9)"));

    m_debugToolBar->Realize();

	// -------------
    m_mgr.AddPane(m_fileToolBar, wxAuiPaneInfo().
		  Name(wxT("file toolbar")).
		  Caption(wxT("File")).
		  ToolbarPane().Top().Row(0).Position(0));
	
    m_mgr.AddPane(m_debugToolBar, wxAuiPaneInfo().
		  Name(wxT("debug toolbar")).
		  Caption(wxT("File")).
		  ToolbarPane().Top().Row(0).Position(1));

	// ----------- wxStyledTextCtrl style
	m_stcStyle = new STCStyle();
	m_stcStyle->SetKeyWords("auto const double float int short struct unsigned break continue else for long signed switch void case default enum goto register sizeof typedef volatile char do extern if return static union while asm dynamic_cast namespace reinterpret_cast try bool explicit new static_cast typeid catch false operator template typename class friend private this using const_cast inline public throw virtual delete mutable protected true wchar_t and bitand compl not_eq or_eq xor_eq and_eq bitor not or xor cin endl INT_MIN iomanip main npos std cout include INT_MAX iostream MAX_RAND NULL string");
			
	wxString stylePath = exePath + "/style";
	m_stcStyle->Scan(stylePath);
	
	// ----------- console
	m_console = new Console(this, this, wxID_ANY,
			wxDefaultPosition, wxSize(600,150));

    m_mgr.AddPane(m_console, wxAuiPaneInfo().
		Name(wxT("console")).
		Caption(wxT("Console")).
		Bottom().CloseButton(true).MaximizeButton(false));
    
	// ----------- watch variable
    m_watch = new Watch(
		this, this, wxID_ANY|wxNO_BORDER,
		wxDefaultPosition, wxSize(200,200));
    m_watch->SetFont(GetDockArt()->GetFont(wxAUI_DOCKART_CAPTION_FONT));
	
    m_mgr.AddPane(m_watch, wxAuiPaneInfo().
		Name(wxT("watch")).
		Caption(wxT("Watch")).
		Left().Layer(1).CloseButton(true).MaximizeButton(false));
	m_mgr.GetPane("watch").Show(false);
	
	// ----------- register
    m_register = new Register(
		this, this, wxID_ANY|wxNO_BORDER,
		wxDefaultPosition, wxSize(200,200));
    m_register->SetFont(GetDockArt()->GetFont(wxAUI_DOCKART_CAPTION_FONT));
	
    m_mgr.AddPane(m_register, wxAuiPaneInfo().
		Name(wxT("register")).
		Caption(wxT("Register")).
		Left().Layer(1).CloseButton(true).MaximizeButton(false));
	m_mgr.GetPane("register").Show(false);
	
	// ----------- memory
    m_memory = new Memory(
		this, this, wxID_ANY|wxNO_BORDER,
		wxDefaultPosition, wxSize(200,200));
    m_memory->SetFont(GetDockArt()->GetFont(wxAUI_DOCKART_CAPTION_FONT));
	
    m_mgr.AddPane(m_memory, wxAuiPaneInfo().
		Name(wxT("memory")).
		Caption(wxT("Memory")).
		Left().Layer(1).CloseButton(true).MaximizeButton(false));
	m_mgr.GetPane("memory").Show(false);
	
	// ----------- call stack
    m_callStack = new CallStack(
		this, this, wxID_ANY|wxNO_BORDER,
		wxDefaultPosition, wxSize(200,120));
    m_callStack->SetFont(GetDockArt()->GetFont(wxAUI_DOCKART_CAPTION_FONT));
	
    m_mgr.AddPane(m_callStack, wxAuiPaneInfo().
		Name(wxT("callstack")).
		Caption(wxT("Backtrace")).
		Top().Layer(0).CloseButton(true).MaximizeButton(false));
	
	// ----------- breakpoint
    m_breakpoint = new Breakpoint(
		this, this, wxID_ANY|wxNO_BORDER,
		wxDefaultPosition, wxSize(200,120));
    m_breakpoint->SetFont(GetDockArt()->GetFont(wxAUI_DOCKART_CAPTION_FONT));
    
	m_mgr.AddPane(m_breakpoint, wxAuiPaneInfo().
		Name(wxT("breakpoint")).
		Caption(wxT("Breakpoint")).
		Top().Layer(0).CloseButton(true).MaximizeButton(false));
	
	// --------------------------
	m_debugToolBar->EnableTool(ID_START, true);
	m_debugToolBar->EnableTool(ID_ABORT, false);

	m_debugToolBar->EnableTool(ID_GO, false);
	m_debugToolBar->EnableTool(ID_STEP_INTO, false);
	m_debugToolBar->EnableTool(ID_STEP_OUT, false);
	m_debugToolBar->EnableTool(ID_STEP_OVER, false);
	m_debugToolBar->EnableTool(ID_RUN_TO_CURSOR, false);
	
	m_debugToolBar->EnableTool(ID_STOP, false);
	
	// ----------
	m_menuBar->Enable(ID_START, true);
	m_menuBar->Enable(ID_ABORT, false);

	m_menuBar->Enable(ID_GO, false);
	m_menuBar->Enable(ID_STEP_INTO, false);
	m_menuBar->Enable(ID_STEP_OUT, false);
	m_menuBar->Enable(ID_STEP_OVER, false);
	m_menuBar->Enable(ID_RUN_TO_CURSOR, false);
	
	m_menuBar->Enable(ID_STOP, false);
	
	// ------
#ifdef __MINGW32__
	wxAuiMDIClientWindow* nb = GetClientWindow();
	nb->SetFont(GetDockArt()->GetFont(wxAUI_DOCKART_CAPTION_FONT));
#endif
	
	wxString m_perspectives = m_config->GetProperty("GEOMETRY", "value");
	if ( m_perspectives.Length() > 0 )
	{
		m_mgr.LoadPerspective(m_perspectives);
	}
	
	m_mgr.Update();
}

MyGdbFrame::~MyGdbFrame()
{
	m_config->SetProperty("GEOMETRY", "value", m_mgr.SavePerspective());
	m_config->Save();

    m_mgr.UnInit();
}

void MyGdbFrame::OnSize(wxSizeEvent& event)
{
}

void MyGdbFrame::OnUpdateUI(wxUpdateUIEvent& WXUNUSED(event))
{
}

wxAuiDockArt* MyGdbFrame::GetDockArt()
{
    return m_mgr.GetArtProvider();
}

void MyGdbFrame::OnExit(wxCommandEvent& WXUNUSED(event))
{
	m_config->SetProperty("GEOMETRY", "value", m_mgr.SavePerspective());
	m_config->Save();

	m_mgr.UnInit();
	exit(0);
}

void MyGdbFrame::OnCloseApp(wxCloseEvent& WXUNUSED(event))
{
	m_config->SetProperty("GEOMETRY", "value", m_mgr.SavePerspective());
	m_config->Save();

	m_mgr.UnInit();
	exit(0);
}

void MyGdbFrame::OnStart(wxCommandEvent& WXUNUSED(event))
{
	wxString value = 
		m_config->GetProperty("INVOKE/GDBLOCATION", "value");
	if ( value.Length() <= 0 )
	{
		wxMessageDialog(this, 
			wxT("No specified 'GDB Location'\nInvoke -> Invoke Application"), 
			"MyGDB", wxOK|wxICON_WARNING).ShowModal();
		return;
	}

	value = m_config->GetProperty("INVOKE/EXECUTABLELOCATION", "value");
	if ( value.Length() <= 0 )
	{
		wxMessageDialog(this, 
			wxT("No specified 'Executable File'\nInvoke -> Invoke Application"), 
			"MyGDB", wxOK|wxICON_WARNING).ShowModal();
		return;
	}
	
	value = m_config->GetProperty("INVOKE/SOURCELOCATION", "value");
	if ( value.Length() <= 0 )
	{
		wxMessageDialog(this, 
			wxT("No specified 'Source Location'\nInvoke -> Invoke Application"), 
			"MyGDB", wxOK|wxICON_WARNING).ShowModal();
		return;
	}

	m_debugger = new Debugger(this);

	if ( m_debugger->Start() == false )
		m_debugger->Abort();
}

void MyGdbFrame::OnAbort(wxCommandEvent& WXUNUSED(event))
{
	if ( m_started )
		m_debugger->Abort();
}

void MyGdbFrame::OnStop(wxCommandEvent& WXUNUSED(event))
{
	if ( m_started )
		m_debugger->Stop();
}

void MyGdbFrame::OnStepOver(wxCommandEvent& WXUNUSED(event))
{
	if ( m_started )
	{
		m_console->Puts("\n");
		m_debugger->Write("next");
	}
}

void MyGdbFrame::OnStepInto(wxCommandEvent& WXUNUSED(event))
{
	if ( m_started )
	{
		m_console->Puts("\n");
		m_debugger->Write("si");
	}
}

void MyGdbFrame::OnStepOut(wxCommandEvent& WXUNUSED(event))
{
	if ( m_started )
	{
		m_console->Puts("\n");
		m_debugger->Write("finish");
	}
}

void MyGdbFrame::OnGo(wxCommandEvent& WXUNUSED(event))
{
	if ( m_started )
	{
		m_console->Puts("\n");
		m_debugger->Write("continue");
	}
}

void MyGdbFrame::AddBreakpoint(wxString path, int line)
{
	if ( m_started )
	{
		m_console->Puts("\n");
		m_debugger->Write(wxString::Format("break \"%s\":%d", 
					path.c_str(), line));
	}
}

void MyGdbFrame::RemoveBreakpoint(wxString path, int line)
{
	if ( m_started )
	{
		wxString breakpoints = 
			m_config->GetProperty("DEBUG/BREAKPOINT", "value");

		wxStringTokenizer tkz(breakpoints, ";");
		while ( tkz.HasMoreTokens() ) 
		{
			wxString point = tkz.GetNextToken();

			wxString _num, _address, _source, _line;

			wxStringTokenizer tkz2(point, "|");
			if ( tkz2.CountTokens() == 4 )
			{
				_num = tkz2.GetNextToken();
				_address = tkz2.GetNextToken();
				_source = tkz2.GetNextToken();
				_line = tkz2.GetNextToken();

				if ( path == _source && line == atoi(_line.c_str()) )
				{
					wxString cmd = wxString::Format("delete %d", 
						atoi(_num.c_str()));
					m_debugger->Eval(cmd);
				}
			}
		}

		m_breakpoint->Build();
	}
}


void MyGdbFrame::OnRunToCursor(wxCommandEvent& WXUNUSED(event))
{
	if ( m_started )
	{
		CodeEditor *editor = (CodeEditor *)GetActiveChild();
		if ( editor ) 
		{
			wxString path = editor->GetFullPath();
			int line = editor->GetSTC()->GetCurrentLine();

			m_console->Puts("\n");
			m_debugger->Write(
				wxString::Format("advance %s:%d", 
				path.c_str(), line+1));
		}
	}
}

CodeEditor *MyGdbFrame::OpenSource(wxString path)
{
	path.Replace(wxT("\\"), wxT("/"), true);

	std::vector<CodeEditor*> editors;
	GetAllEditors(editors);
	
	CodeEditor *editor;

	bool exist = false;
	for (size_t i = 0; i < editors.size(); i++) 
	{
		wxString _path = editors[i]->GetFullPath();
		if ( _path == path )
		{
			editor = editors[i];
			exist=true;
			break;
		}
	}
	
	if ( exist == false )
	{
		editor = new CodeEditor(this, this, wxID_ANY, path);
		editor->Open(path);
		editor->Layout();
		editor->ShowBreakpoint();
	}

	editor->Activate();
	editor->SetFocus();


	return editor;
}

void MyGdbFrame::OnOpenFile(wxCommandEvent& WXUNUSED(event))
{
	wxString path = m_config->GetProperty("INVOKE/SOURCELOCATION", "value");

    wxFileDialog dlg(this,
		wxT("Open Source"), path,
		wxEmptyString,
		wxT("C/C++ Files (*.c;*.cpp)|*.c;*.cpp|Header Files (*.h)|*.h|All Files (*)|*")
	);
    dlg.CentreOnParent();

    if (dlg.ShowModal() == wxID_OK)
	{
		OpenSource(dlg.GetPath());
	}
}

void MyGdbFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
	About *dlg = new About(this, this);
	//dlg->SetAppName(wxTheApp->GetAppName());
	dlg->SetAppName("MyGDB");
	dlg->SetVersion("dev-20100722");

	wxString copy = wxT("MyGDB is distributed under the GNU GPL-v3\n\n");
	copy = copy + wxT("inhak.min@gmail.com");
	dlg->SetCopyright(copy);

	dlg->SetCustomBuildInfo(wxString::Format(wxT("%s. %s"),
		About::GetBuildInfo(About::wxBUILDINFO_LONG).GetData(),
		_("\nPowered by wxWidgets")));
	dlg->SetHeaderBitmap(wxGetBitmapFromMemory(
				about_bitmap, sizeof(about_bitmap)));
	dlg->ApplyInfo();
	dlg->ShowModal();
	dlg->Destroy();
}

void MyGdbFrame::OnInvokeApp(wxCommandEvent& WXUNUSED(event))
{
    InvokeApp dlg(this, this, wxID_ANY, wxT("Invoke Application"));
	wxSize size = dlg.GetBestSize();
	dlg.SetSize(wxSize(400, size.GetHeight()));
    dlg.CentreOnParent();

    if ( dlg.ShowModal() == wxID_OK ) 
	{
		wxString executableFile = dlg.GetExecutableFile();
		wxString sourceLocation = dlg.GetSourceLocation();
		wxString gdbLocation = dlg.GetGDBLocation();
		wxString args = dlg.GetArguments();

		m_config->SetProperty("INVOKE/EXECUTABLELOCATION", 
				"value", executableFile);
		m_config->SetProperty("INVOKE/SOURCELOCATION", 
				"value", sourceLocation);
		m_config->SetProperty("INVOKE/GDBLOCATION", 
				"value", gdbLocation);
		m_config->SetProperty("INVOKE/ARGS", 
				"value", args);
		m_config->Save();
	}
}

void MyGdbFrame::GetAllEditors(std::vector<CodeEditor*> &editors)
{
	editors.clear();
	wxAuiNotebook* nb = GetNotebook();
	for (size_t i = 0; i < nb->GetPageCount(); i++) {
		CodeEditor *editor = dynamic_cast<CodeEditor*>(nb->GetPage(i));
		if (editor) {
			editors.push_back(editor);
		}
	}
}

void MyGdbFrame::OnViewConsole(wxCommandEvent& WXUNUSED(event))
{
	m_mgr.GetPane("console").Show(true);
	m_mgr.Update();
}

void MyGdbFrame::OnViewWatch(wxCommandEvent& WXUNUSED(event))
{
	m_mgr.GetPane("watch").Show(true);
	m_mgr.Update();
}

void MyGdbFrame::OnViewCallStack(wxCommandEvent& WXUNUSED(event))
{
	m_mgr.GetPane("callstack").Show(true);
	m_mgr.Update();
}

void MyGdbFrame::OnViewBreakpoint(wxCommandEvent& WXUNUSED(event))
{
	m_mgr.GetPane("breakpoint").Show(true);
	m_mgr.Update();
}

void MyGdbFrame::OnViewRegister(wxCommandEvent& WXUNUSED(event))
{
	m_mgr.GetPane("register").Show(true);
	m_mgr.Update();
}

void MyGdbFrame::OnViewMemory(wxCommandEvent& WXUNUSED(event))
{
	m_mgr.GetPane("memory").Show(true);
	m_mgr.Update();
}
